package com.gct.tools.etlcamelhuge.routeconfig;

import org.apache.camel.*;
import org.apache.camel.builder.RouteBuilder;
//import org.apache.rocketmq.common.message.Message;
import org.apache.camel.model.ExpressionNode;
import org.apache.camel.model.ProcessorDefinition;
import org.apache.camel.model.RouteDefinition;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * class name: CamelJDBCConfiguration
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/4/14 下午3:16
 */
@Configuration
public class CamelJDBCConfiguration {


    public String getDate() {
        return LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
    }

    @Bean
    public RouteBuilder routeBuilderWithOracle1() {
        return new RouteBuilder() {
            private SortedSet<String> organization;
            private Map<String, Integer> orgIDs = new HashMap<>();
            private int orgID;
            private Map<Object, Object> orgIdPreList;
            private Integer org;

            public ProcessorDefinition<ExpressionNode> setSysControlBody(RouteDefinition route) {
                return route.setBody(simple("select well_id from centralbase.sys_access_well_control  where  access_status='1'  "))
                        .to("jdbc:centralbase")
                        .split(body()).process(exchange -> {
                            HashMap body = exchange.getIn().getBody(HashMap.class);
                            exchange.getIn().setHeader("well_id", body.get("well_id"));
                        });
            }

            @Override
            public void configure() throws Exception {


                from("timer:update-wellControl?period=3600000")
                        .routeId("update-wellControl")
                        .setBody(simple("select scc.well_id,wo.well_common_name,op.org_id from centralbase.sys_access_well_control scc\n" +
                                "left join centralbase.cb_cd_well_source wo on scc.well_id = wo.well_id\n" +
                                "left join centralbase.cb_pc_organization op on wo.org_id = op.org_id\n" +
                                "where scc.access_status ='1' "))
                        .to("jdbc:centralbase")
                        .split(body())
                        .split(body()).process(exchange -> {
                            HashMap<String, Object> body = exchange.getIn().getBody(HashMap.class);
                            DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                            String format = LocalDateTime.now().format(df);
                            String yesterday = LocalDateTime.now().minusDays(1).format(df);
                            body.put("yesterday", yesterday);
                            if (body.get("well_common_name") == null || body.get("well_common_name").equals("")) {
                                body.put("remarks", "暂无井信息");
                                body.put("error_id", 0);
                                body.put("updateTime", format);
                            } else if (body.get("org_id") == null || body.get("org_id").equals("")) {
                                body.put("remarks", "暂无机构信息");
                                body.put("error_id", 0);
                                body.put("updateTime", format);
                            } else {
                                body.put("remarks", "");
                                body.put("error_id", 1);
                                body.put("updateTime", format);
                            }

                        })
                        //之前在数据导入时没有自动填写入最后更新时间,会导致新加入的数据无法自动导入
                        .setBody(simple("update centralbase.sys_access_well_control set well_common_name='${body[well_common_name]}'," +
                                "org_id='${body[org_id]}',update_time = '${body[updateTime]}'::timestamp,sgt_last_time='${body[updateTime]}', " +
                                "remarks ='${body[remarks]}' ,error_id ='${body[error_id]}' " +
                                "where well_id ='${body[well_id]} and " +
                                "(sgt_last_time is null or sgt_last_time<'${body[yesterday]}'::timestamp)"))
                        .to("jdbc:centralbase")
                        .end();

                RouteDefinition statusDaily = from("timer:mytimer-insert-statusDaily?period=10800000")
                        .routeId("insert-statusDaily")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        });
                setSysControlBody(statusDaily)
                        .setBody(simple("select  distinct WELL_ID,PROD_DATE,OIL_PRODUCTION_METHOD ,BACK_PRES,MAX_TUBING_PRES,MIN_TUBING_PRES,CASING_PRES" +
                                " from ZY_DXGW.PC_PRO_COM_DAILY " +
                                "where prod_date  = to_date('${header.date}','yyyy-MM-dd') and well_id ='${header.well_id}'"))
                        .to("jdbc:oracle")
                        .split(body()).process(exchange -> {
                            Message in = exchange.getIn();
                            HashMap<String, Object> aRow = in.getBody(HashMap.class);
                            aRow.putIfAbsent("OIL_NOZZLE", "0.0");
                            aRow.putIfAbsent("BACK_PRES", "0.0");
                            if (aRow.get("MAX_TUBING_PRES") == null || aRow.get("MIN_TUBING_PRES") == null)
                                aRow.put("TUBING_PRES", "0.0");
                            else {
                                aRow.put("TUBING_PRES", String.format("%.2f",(Double.parseDouble(aRow.get("MAX_TUBING_PRES").toString())
                                        + Double.parseDouble(aRow.get("MIN_TUBING_PRES").toString())) / 2));
                            }
                            aRow.putIfAbsent("CASING_PRES", "0.0");
                            aRow.putIfAbsent("PUMP_DEPTH", "0.0");
                        })
                        .setBody(simple("insert into centralbase.cb_pc_pro_wellbore_status_daily(well_id,prod_date,oil_prod_method,back_pres,tubing_pres,casing_pres) " +
                                "select '${body[WELL_ID]}','${body[PROD_DATE]}','${body[OIL_PRODUCTION_METHOD]}'," +
                                "'${body[BACK_PRES]}','${body[TUBING_PRES]}','${body[CASING_PRES]}'" +
                                "where NOT EXISTS ( SELECT * FROM centralbase.cb_pc_pro_wellbore_status_daily " +
                                "      WHERE well_id = '${body[WELL_ID]}' and  prod_date = '${body[PROD_DATE]}')"))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:insert-statusDaily ->  centralbase.cb_pc_pro_wellbore_status_daily insert data failed")
                        .end();

                // BS BJ DYM find before data
                RouteDefinition statusDailyDYM = from("timer:mytimer-update-statusDaily-DYM?period=10800000")
                        .routeId("update-statusDaily-DYM")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        });
                setSysControlBody(statusDailyDYM)
                        .setBody(simple("select distinct WELL_ID , PROD_DATE , DYNAMIC_LIQ_LEVEL " +
                                "from ZY_DXGW.PC_PRO_COM_DAILY " +
                                "where  (WELL_ID,PROD_DATE) in" +
                                " (SELECT  WELL_ID,max(PROD_DATE) PROD_DATE  FROM ZY_DXGW.PC_PRO_COM_DAILY" +
                                "    WHERE DYNAMIC_LIQ_LEVEL is not null and WELL_ID='${header.well_id}' group by WELL_ID)"))
                        .to("jdbc:oracle")
                        .split(body())
                        .setBody(simple("update centralbase.cb_pc_pro_wellbore_status_daily set start_pump_liq_level = '${body[DYNAMIC_LIQ_LEVEL]}'  " +
                                "where well_id = '${header.well_id}' and prod_date::date  = '${header.date}' "))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:update-statusDaily-DYM ->  centralbase.cb_pc_pro_wellbore_status_daily update data failed")
                        .end();

                RouteDefinition statusDailyBS = from("timer:mytimer-update-statusDaily-BS?period=10800000")
                        .routeId("update-statusDaily-BS")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        });
                setSysControlBody(statusDailyBS)
                        .setBody(simple("select distinct WELL_ID , PROD_DATE , PUMP_DEPTH " +
                                "from ZY_DXGW.PC_PRO_COM_DAILY " +
                                "where  (WELL_ID,PROD_DATE) in" +
                                " (SELECT  WELL_ID,max(PROD_DATE) PROD_DATE  FROM ZY_DXGW.PC_PRO_COM_DAILY" +
                                "    WHERE PUMP_DEPTH is not null and WELL_ID='${header.well_id}' group by WELL_ID)"))
                        .to("jdbc:oracle")
                        .split(body())
                        .setBody(simple("update centralbase.cb_pc_pro_wellbore_status_daily set pump_depth = '${body[PUMP_DEPTH]}'  " +
                                "where well_id = '${header.well_id}' and prod_date::date  = '${header.date}' "))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:update-statusDaily-BS ->  centralbase.cb_pc_pro_wellbore_status_daily update data failed")
                        .end();

                RouteDefinition statusDailyBJ = from("timer:mytimer-update-statusDaily-BJ?period=10800000")
                        .routeId("update-statusDaily-BJ")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        });
                setSysControlBody(statusDailyBJ)
                        .setBody(simple("select distinct WELL_ID , PROD_DATE , PUMP_DIAMETER " +
                                "from ZY_DXGW.PC_PRO_COM_DAILY " +
                                "where  (WELL_ID,PROD_DATE) in" +
                                " (SELECT  WELL_ID,max(PROD_DATE) PROD_DATE  FROM ZY_DXGW.PC_PRO_COM_DAILY" +
                                "    WHERE PUMP_DIAMETER is not null and WELL_ID='${header.well_id}' group by WELL_ID)"))
                        .to("jdbc:oracle")
                        .split(body())
                        .setBody(simple("update centralbase.cb_pc_pro_wellbore_status_daily set oil_nozzle = '${body[PUMP_DIAMETER]}'  " +
                                "where well_id = '${header.well_id}' and prod_date::date  = '${header.date}' "))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:update-statusDaily-BJ ->  centralbase.cb_pc_pro_wellbore_status_daily update data failed")
                        .end();

                RouteDefinition submergenceDepth = from("timer:mytimer-update-statusDaily-submergenceDepth?period=10800000")
                        .routeId("update-statusDaily-submergenceDepth")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate() + " 00:00:00");
                        });
                setSysControlBody(submergenceDepth)
                        .setBody(simple("select well_id,prod_date,start_pump_liq_level,pump_depth from centralbase.cb_pc_pro_wellbore_status_daily where prod_date = '${header.date}' and well_id='${header.well_id}' "))
                        .to("jdbc:centralbase")
                        .split(body()).process(exchange -> {
                            Message in = exchange.getIn();
                            HashMap<String, Object> aRow = in.getBody(HashMap.class);
                            aRow.put("submergence_depth", null);
                            if (aRow.get("start_pump_liq_level") != null && aRow.get("pump_depth") != null) {
                                double cmd = Double.parseDouble(aRow.get("pump_depth").toString()) - Double.parseDouble(aRow.get("start_pump_liq_level").toString()) / 10;
                                BigDecimal bd = new BigDecimal(cmd);
                                double cmd1 = bd.setScale(1, RoundingMode.HALF_UP).doubleValue();
                                aRow.put("submergence_depth", cmd1);
                            }
                        })
                        .setBody(simple("update centralbase.cb_pc_pro_wellbore_status_daily set submergence_depth = '${body[submergence_depth]}' where well_id = '${body[well_id]}' and prod_date  = '${body[prod_date]}'"))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:update-statusDaily-submergenceDepth ->  centralbase.cb_pc_pro_wellbore_status_daily update data failed")
                        .end();



                RouteDefinition volDaily = from("timer:mytimer-insert-volDaily?period=10800000")
                        .routeId("insert-volDaily")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        });
                setSysControlBody(volDaily)
                        .setBody(simple("select distinct  WELL_ID,PROD_DATE,PROD_TIME, LIQ_PROD_DAILY, OIL_PROD_DAILY,GAS_PROD_DAILY" +
                                ",WATER_CUT, REMARKS from DBA01 where PROD_DATE  = to_date('${header.date}','yyyy-MM-dd')" +
                                " and WELL_ID ='${header.well_id}'"))
                        .to("jdbc:oracle")
                        .split(body()).process(exchange -> {
                            Message in = exchange.getIn();
                            HashMap<String, Object> aRow = in.getBody(HashMap.class);
                            aRow.putIfAbsent("PROD_TIME", "0.0");
                            aRow.putIfAbsent("LIQ_PROD_DAILY", "0.0");
                            aRow.putIfAbsent("OIL_PROD_DAILY", "0.0");
                            aRow.putIfAbsent("GAS_PROD_DAILY", "0.0");
                            aRow.putIfAbsent("WATER_CUT", "0.0");
                            aRow.putIfAbsent("REMARKS", "");
                            aRow.put("SCCW","");
                            aRow.put("RCSL", -1);
                            aRow.put("QYB", -1);
                            aRow.put("SQB", -1);
                            if (aRow.get("GAS_PROD_DAILY") != null && aRow.get("OIL_PROD_DAILY") != null && !aRow.get("OIL_PROD_DAILY").equals("0.0") && aRow.get("OIL_PROD_DAILY") != "0.0") {
                                double qyb = Double.valueOf(aRow.get("GAS_PROD_DAILY").toString()) / Double.valueOf(aRow.get("OIL_PROD_DAILY").toString());
                                if (!Double.isNaN(qyb) && !Double.isInfinite(qyb)) {
                                    BigDecimal bd = new BigDecimal(qyb);
                                    double d1 = bd.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                    aRow.put("QYB", d1);
                                }
                            }
                            if (aRow.get("LIQ_PROD_DAILY") != null && aRow.get("WATER_CUT") != null && !aRow.get("WATER_CUT").equals("0.0") && aRow.get("WATER_CUT") != "0.0") {
                                double rcsl = (Double.valueOf(aRow.get("LIQ_PROD_DAILY").toString()) * Double.valueOf(aRow.get("WATER_CUT").toString())) / 100;
                                if (!Double.isNaN(rcsl) && !Double.isInfinite(rcsl)) {
                                    BigDecimal bd = new BigDecimal(rcsl);
                                    double d1 = bd.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                    aRow.put("RCSL", d1);
                                }
                            }
                            if (aRow.get("GAS_PROD_DAILY") != null && aRow.get("RCSL") != null && !aRow.get("GAS_PROD_DAILY").equals("0.0") && aRow.get("GAS_PROD_DAILY") != "0.0") {
                                double sqb = Double.valueOf(aRow.get("RCSL").toString()) / Double.valueOf(aRow.get("GAS_PROD_DAILY").toString());
                                if (!Double.isNaN(sqb) && !Double.isInfinite(sqb)) {
                                    BigDecimal bd = new BigDecimal(sqb);
                                    double d1 = bd.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                    aRow.put("SQB", d1);
                                }
                            }
                            if (!aRow.containsKey("SMD")) {
                                aRow.put("SMD", 1);
                            }
                            if (!aRow.containsKey("YMD")) {
                                aRow.put("YMD", 0.85);
                            }
                        })
                        .setBody(simple("insert into centralbase.cb_pc_pro_wellbore_vol_daily(well_id,prod_date,prod_time,liq_prod_daily,oil_prod_daily,gas_prod_daily,water_cut,remarks,gas_oil_ratio,water_prod_daily,water_gas_ratio,surface_crude_water_density,surface_crude_oil_density,stim_content) " +
                                "select '${body[WELL_ID]}','${body[PROD_DATE]}','${body[WATER_CUT]}','${body[LIQ_PROD_DAILY]}','${body[OIL_PROD_DAILY]}','${body[GAS_PROD_DAILY]}','${body[WATER_CUT]}','${body[REMARKS]}','${body[QYB]}','${body[RCSL]}','${body[SQB]}','${body[SMD]}','${body[YMD]}' ,'${body[SCCW]}'" +
                                "where NOT EXISTS ( SELECT * FROM centralbase.cb_pc_pro_wellbore_vol_daily WHERE well_id = '${body[WELL_ID]}' and  prod_date = '${body[PROD_DATE]}' )"))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:insert-volDaily ->  centralbase.cb_pc_pro_wellbore_vol_daily insert data failed")
                        .endDoTry()
                        .setBody(simple("update centralbase.cb_pc_pro_wellbore_vol_daily set  water_prod_daily =null where water_prod_daily = -1 and prod_date = '${header.date}' "))
                        .to("jdbc:centralbase")
                        .setBody(simple("update centralbase.cb_pc_pro_wellbore_vol_daily set  gas_oil_ratio =null where gas_oil_ratio = -1 and prod_date = '${header.date}' "))
                        .to("jdbc:centralbase")
                        .setBody(simple("update centralbase.cb_pc_pro_wellbore_vol_daily set  water_gas_ratio =null where water_gas_ratio = -1 and prod_date = '${header.date}' "))
                        .to("jdbc:centralbase")
                        .end();

                RouteDefinition volDailyLiqProdDaily = from("timer:mytimer-update-volDaily-liq_prod_daily?period=10800000")
                        .routeId("update-volDaily-liq_prod_daily")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        });
                setSysControlBody(volDailyLiqProdDaily)
                        .setBody(simple("select distinct  WELL_ID,PROD_DATE,WATER_CUT, LIQ_PROD_DAILY,OIL_PROD_DAILY,GAS_PROD_DAILY,WATER_CUT, REMARKS from DBA01 where PROD_DATE  = to_date('${header.date}','yyyy-MM-dd')and WELL_ID ='${header.well_id}' and qyrq is not null "))
                        .to("jdbc:oracle")
                        .split(body()).process(exchange -> {
                            Message in = exchange.getIn();
                            HashMap<String, Object> aRow = in.getBody(HashMap.class);
                            if (aRow.get("WATER_CUT") == null) aRow.put("WATER_CUT", "0.0");
                            if (aRow.get("LIQ_PROD_DAILY") == null) aRow.put("LIQ_PROD_DAILY", "0.0");
                            if (aRow.get("OIL_PROD_DAILY") == null) aRow.put("OIL_PROD_DAILY", "0.0");
                            if (aRow.get("GAS_PROD_DAILY") == null) aRow.put("GAS_PROD_DAILY", "0.0");
                            if (aRow.get("WATER_CUT") == null) aRow.put("WATER_CUT", "0.0");
                            if (aRow.get("REMARKS") == null) aRow.put("REMARKS", "");
                            aRow.put("RCSL", -1);
                            aRow.put("QYB", -1);
                            aRow.put("SQB", -1);
                            if (aRow.get("GAS_PROD_DAILY") != null && aRow.get("OIL_PROD_DAILY") != null && !aRow.get("OIL_PROD_DAILY").equals("0.0") && aRow.get("OIL_PROD_DAILY") != "0.0") {
                                double qyb = Double.valueOf(aRow.get("GAS_PROD_DAILY").toString()) / Double.valueOf(aRow.get("OIL_PROD_DAILY").toString());
                                if (!Double.isNaN(qyb) && !Double.isInfinite(qyb)) {
                                    BigDecimal bd = new BigDecimal(qyb);
                                    double d1 = bd.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                    aRow.put("QYB", d1);
                                }
                            }
                            if (aRow.get("LIQ_PROD_DAILY") != null && aRow.get("WATER_CUT") != null && !aRow.get("WATER_CUT").equals("0.0") && aRow.get("WATER_CUT") != "0.0") {
                                double rcsl = (Double.valueOf(aRow.get("LIQ_PROD_DAILY").toString()) * Double.valueOf(aRow.get("WATER_CUT").toString())) / 100;
                                if (!Double.isNaN(rcsl) && !Double.isInfinite(rcsl)) {
                                    BigDecimal bd = new BigDecimal(rcsl);
                                    double d1 = bd.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                    aRow.put("RCSL", d1);
                                }
                            }
                            if (aRow.get("GAS_PROD_DAILY") != null && aRow.get("RCSL") != null && !aRow.get("GAS_PROD_DAILY").equals("0.0") && aRow.get("GAS_PROD_DAILY") != "0.0") {
                                double sqb = Double.valueOf(aRow.get("RCSL").toString()) / Double.valueOf(aRow.get("GAS_PROD_DAILY").toString());
                                if (!Double.isNaN(sqb) && !Double.isInfinite(sqb)) {
                                    BigDecimal bd = new BigDecimal(sqb);
                                    double d1 = bd.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                    aRow.put("SQB", d1);
                                }
                            }
                            if (!aRow.containsKey("SMD")) {
                                aRow.put("SMD", 1);
                            }
                            if (!aRow.containsKey("YMD")) {
                                aRow.put("YMD", 0.85);
                            }
                        })
                        .setBody(simple("update centralbase.cb_pc_pro_wellbore_vol_daily set prod_time = '${body[WATER_CUT]}' ,liq_prod_daily='${body[LIQ_PROD_DAILY]}' ,oil_prod_daily ='${body[OIL_PROD_DAILY]}' ,gas_prod_daily ='${body[GAS_PROD_DAILY]}' " +
                                ",water_cut='${body[WATER_CUT]}' ,remarks='${body[REMARKS]}' ,gas_oil_ratio='${body[QYB]}' " +
                                ",water_prod_daily='${body[RCSL]}' ,water_gas_ratio='${body[SQB]}',surface_crude_water_density='${body[SMD]}'" +
                                ",surface_crude_oil_density= '${body[YMD]}' " +
                                "where well_id = '${body[WELL_ID]}' and prod_date ='${body[PROD_DATE]}' "))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:update-volDaily-liq_prod_daily ->  centralbase.cb_pc_pro_wellbore_vol_daily update data failed")
                        .end();

               /* RouteDefinition runTime = from("timer:mytimer-update-runTime-strokeAndFrequency?period=10800000")
                        .routeId("update-runTime-strokeAndFrequency")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        });
                setSysControlBody(runTime)
                        .setBody(simple("select distinct  WELL_ID,DYNA_CREATE_TIME,STROKE,FREQUENCY from DBA01 where PROD_DATE  = to_date('${header.date}','yyyy-MM-dd') and WELL_ID ='${header.well_id}'"))
                        .to("jdbc:oracle")
                        .split(body())
                        .setBody(simple("insert into centralbase.cb_temp_well_mech_runtime(well_id,prod_date,stroke_length,stroke_frequency)" +
                                "select '${body[WELL_ID]}','${body[PROD_DATE]}','${body[STROKE]}','${body[FREQUENCY]}'" +
                                "where NOT EXISTS (SELECT * FROM centralbase.cb_temp_well_mech_runtime WHERE well_id = '${body[WELL_ID]}' and  prod_date = '${body[PROD_DATE]}' )"))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:update-runTime-strokeAndFrequency ->  centralbase.cb_temp_well_mech_runtime update data failed")
                        .end();*/
            }

            ;
        };


    }
}
